package com.jobs.controller;

import com.jobs.common.Result;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * 文件上传下载接口
 */
@RestController
@RequestMapping("/updown")
public class UpdownController {

    @Value("${upload.path}")
    private String uploadPath;

    /**
     * 获取已经上传的文件列表
     */
    @GetMapping("/list")
    public Result<List<String>> getFileList() {
        File dir = new File(uploadPath);
        if (!dir.exists()) {
            dir.mkdirs();
        }

        //获取文件夹下的文件列表
        File[] flist = dir.listFiles();
        List<String> sss = Arrays.stream(flist).filter(s -> s.isFile())
                .sorted((s1, s2) -> {
                    //按照文件修改时间倒序排列，后上传的文件，排在前面
                    return (int) (s2.lastModified() - s1.lastModified());
                })
                .map(s -> s.getName()).collect(Collectors.toList());
        return Result.success(sss);
    }

    /**
     * 上传文件接口
     *
     * @param file 选择一个文件
     */
    @PostMapping("/upload")
    public Result<String> upload(MultipartFile file) {

        //如果你想要上传的文件，使用一个随机的文件名称时，可以使用这些代码
        //原始文件名
        String originalName = file.getOriginalFilename();
        //获取后缀名
        String extName = FilenameUtils.getExtension(originalName);
        String newName = UUID.randomUUID().toString().replace("-", "") + "." + extName;
        File dir = new File(uploadPath);
        if (!dir.exists()) {
            dir.mkdirs();
        }

        try {
            file.transferTo(new File(uploadPath + newName));
        } catch (IOException e) {
            e.printStackTrace();
        }

        return Result.success(newName);

        //这里保留上传后的文件名跟原始文件名一致，服务器上如果文件已存在，则直接覆盖
        //原始文件名
        /*String originalName = file.getOriginalFilename();
        try {
            File dir = new File(uploadPath);
            if (!dir.exists()) {
                dir.mkdirs();
            }
            file.transferTo(new File(uploadPath + originalName));
        } catch (Exception ex) {
            return Result.fail(0, ex.getMessage());
        }

        return Result.success("上传成功");*/
    }

    /**
     * 下载文件接口
     *
     * @param name 要下载的文件名称
     */
    @GetMapping("/download")
    public void download(String name, HttpServletResponse response) throws UnsupportedEncodingException {

        //下载文件的响应类型，这里统一设置成了文件流
        //你可以根据自己所提供下载的文件类型，使用不同的响应 mime 类型
        response.setContentType("application/octet-stream;charset=utf-8");
        //设置下载弹出框中默认显示的文件名称，如果指定中文名称的话，需要转成 iso8859-1 编码，解决乱码问题
        String fileName = new String(name.getBytes(), "iso8859-1");
        response.addHeader("Content-Disposition", "attachment;filename=" + fileName);

        try (BufferedInputStream bis = new BufferedInputStream(new FileInputStream(uploadPath + fileName))) {
            ServletOutputStream outputStream = response.getOutputStream();
            byte[] bArr = new byte[1024];
            int len;
            while ((len = bis.read(bArr)) != -1) {
                outputStream.write(bArr, 0, len);
            }
        } catch (Exception ex) {
            return;
        }
    }

    /**
     * 删除文件接口
     *
     * @param name 要删除的文件名称
     */
    @GetMapping("/delete")
    public Result<String> delete(String name) {
        File file = new File(uploadPath + name);
        if (!file.exists()) {
            return Result.fail(0, name + "，已经不存在，无法删除");
        }

        try {
            file.delete();
            return Result.success(name + "，删除成功");
        } catch (Exception ex) {
            return Result.fail(0, ex.getMessage());
        }
    }

    /**
     * 清空上传的文件
     */
    @GetMapping("/clear")
    public Result<String> clear() {
        File dir = new File(uploadPath);
        if (!dir.exists()) {
            dir.mkdirs();
        }

        //获取文件夹下的文件列表
        File[] flist = dir.listFiles();
        for (File f : flist) {
            if (f.isFile()) {
                //删除文件
                f.delete();
            } else {
                //删除文件夹
                deleteDirectory(f);
            }
        }

        return Result.success("清空删除成功");
    }

    /**
     * 递归删除一个文件夹，以及文件夹下的所有内容
     */
    private void deleteDirectory(File dir) {
        if (dir.exists()) {
            if (dir.isDirectory()) {
                File[] flist = dir.listFiles();
                for (File f : flist) {
                    if (f.isDirectory()) {
                        deleteDirectory(f);
                    }
                    f.delete();
                }
            }
            dir.delete();
        }
    }
}
